<?php
/**
 * Created by PhpStorm.
 * User: João
 * Date: 21/06/2015
 * Time: 00:17
 */

class HourlyBilling implements BillingType {
    private $_baseRate;

    /**
     * Returns the amount to charge a client
     * @param array $billingData
     * @throws Exception
     * @return float
     */
    public function calculate($billingData = array())
    {
        /**
         * Worked hours are reported daily (hence the array of days)
         * If worked over 8 hours in a specific day the rate is 50% higher for that specific day
         * Billing occur weekly (which is why the 'days' array has to have 5 elements)
         */
        if (!isset($billingData['days']) || count($billingData['days']) != 5)
            throw new Exception('Invalid billing data provided.
                                You need to specify an array of 5 days (one week) with the amount of hours worked per day.');
        $total = 0;
        foreach ($billingData['days'] as $day) {
            if ($day > 8) {
                $total += $day * 1.5 * $this->getBaseRate();
            } else {
                $total += $day * $this->getBaseRate();
            }
        }
        return $total;
    }

    /**
     * Defines the base value to use as rate for later calculations
     * @param float $value
     */
    public function setBaseRate($value) {
        $this->_baseRate = $value;
    }

    /**
     * @return float
     */
    public function getBaseRate() {
        return $this->_baseRate;
    }
}