<?php
/**
 * Created by PhpStorm.
 * User: João
 * Date: 21/06/2015
 * Time: 01:24
 */

class Client {
    private $_billingType;
    private $_paymentMethod;
    private $_email;
    private $_name;
    public function __construct($name, $email)
    {
        $this->setName($name);
        $this->setEmail($email);
    }

    /**
     * GETTERS AND SETTERS
     *
     *
     * @param BillingType $billingType
     */
    public function setBillingType($billingType)
    {
        $this->_billingType = $billingType;
    }

    /**
     * @return BillingType
     */
    public function getBillingType()
    {
        return $this->_billingType;
    }

    /**
     * @param PaymentMethod $paymentOption
     */
    public function setPaymentMethod($paymentOption)
    {
        $this->_paymentMethod = $paymentOption;
    }

    /**
     * @return PaymentMethod
     */
    public function getPaymentMethod()
    {
        return $this->_paymentMethod;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->_name = $name;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->_email = $email;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->_email;
    }

    /**
     * USER FUNCTIONS
     */
    /**
     * @param float $value
     * @throws Exception
     */
    public function setBillingRate($value)
    {
        $billingType = $this->getBillingType();
        if ($billingType == null)
            throw new Exception('No billing type has been defined for ' . $this->getName());
        $billingType->setBaseRate($value);
    }

    /**
     * Bills the client.
     * Calculates the amount, charges it and notifies the client via email
     * @param array $billingData
     * @throws Exception
     */
    public function bill($billingData = array())
    {
        $amountToCharge = $this->getBillingType()->calculate($billingData);
        /**
         * Get the method and charge the client
         */
        $paymentMethod = $this->getPaymentMethod();
        if ($paymentMethod == null)
            throw new Exception('No payment method has been defined for ' . $this->getName());
        $paymentMethod->process($amountToCharge, $this);

        /**
         * Notify the client via email
         */
        $readableAmount = number_format($amountToCharge);
        CommunicationsHelper::sendEmail($this->getEmail(),
                                        'Your latest receipt from Fancy Law Firm',
                                        'Hi there ' . $this->getName() . ',<br />Consider this an invoice from' .
                                        ' Fancy Law Firm totalling ' . $readableAmount . '£');
    }
}